/**************************************************************************/
/*                              Ballbuster                                */
/*                             By Eric Scott                              */
/*                                                                        */
/*                        COPYRIGHT 1989 BY ST-LOG                        */
/*                                                                        */
/*                          Written in Megamax C                          */
/*                     Graphics developed with DEGAS Elite                */
/*                 Sounds developed with the GIST Sound Editor            */
/**************************************************************************/

#include <gemdefs.h>
#include <osbind.h>
#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>

#define MOUSE_ON v_show_c(handle, 0) /* Turn mouse on. */
#define MOUSE_OFF v_hide_c(handle)   /* Turn mouse off. */

#define TRUE   1   /* TRUE flag. */
#define FALSE  0   /* FALSE flag. */
#define MUNCH  2   /* For when the demon is eating the crystal ball. */   
#define PLAY   1   /* Play the game flag. */
#define QUIT   2   /* Quit the game flag. */
#define BOTH   3   /* Both mouse buttons pressed. */

#define UP    1  /* ball track up (also flag for demon movement), */
#define UAL   2  /* ball track up and left, */
#define UAR   3  /* ball track up and right, */
#define LEFT  4  /* ball track left (also flag for demon position), */
#define RIGHT 5  /* ball track right (also flag for demon position), */
#define DOWN  6  /* ball track down (also flag for demon movement), */
#define DAL   7  /* ball track down and left, */
#define DAR   8  /* ball track down and right. */

#define BLANK  0  /* The values contained in the original_globe */
#define BLUE   1  /* and current_globe arrays.  Used in drawing */
#define GREEN  2  /* the globe and in the scoring routines.     */
#define RED    3
#define LBLUE  4
#define VIOLET 5
#define YELLOW 6

/******************  The standard gem arrays and globals  *****************/

int work_in[11],work_out[57],pxarray[10],contrl[12],intin[128],intout[128];
int ptsin[128],ptsout[128];
int handle, dummy;

/**************************************************************************/

/*** The globals for the width, height, x & y coordinates of the sprites **/

int sprite_w = 16, /* Width in pixels of the 16*16 sprites. */
    sprite_h = 15, /* Height-1 of the 16*16 sprites. */
    ball_h = 12,   /* Height-1 of the crystal ball sprites. */
    ball_x,        /* x coordinate for crystal ball sprites. */
    ball_y,        /* y coordinate for crystal ball sprites. */
    top_of_spring, /* y coordinate of spring head. */
    demon_w = 32,  /* Width in pixels of the demon sprites. */
    demon1_h = 39, /* Height-1 of the closed-mouth demon. */
    demon2_h = 44, /* Height-1 of the open-mouth demon. */
    demon_y,       /* y coordinate for demon. */
    mage_w = 32,  /* Width in pixels of mage sprites. */
    mage1_h = 37, /* Height-1 of mage-throwing-ball sprite. */
    mage2_h = 30, /* Height-1 of mage-holding-ball sprite. */
    mage3_h = 36, /* Height-1 of gosh-how-did-I-miss-that-one sprite. */
    mage_x,       /* x coordinate of mage sprites. */
    mage1_y = 159, /* y coordinate for mage-throwing-ball sprite. */
    mage2_y = 167, /* y coordinate for mage-holding-ball sprite. */
    mage3_y = 161, /* y coordinate for g-h-d-I-m-t-o sprite. */
    globe_w = 176, /* Width in pixels of globe "cookie cutter". */
    globe_h = 139; /* Height-1 of globe "cookie cutter". */

/**************************************************************************/

/***********  Other globals to keep track of things in the game  **********/

int max_speed = 8, /* The maximum speed of the ball in certain cases. */
    max_mage = 26, /* The maximum speed of the mage. */
    ball_trak, /* The direction the crystal ball is going. */
    b_speed,   /* The speed of the crystal ball. */
    n_balls,   /* The number of crystal balls you have left. */
    bonus_ball, /* Keeps track of score for extra ball every 5000 pts. */
    gold_ball,  /* Value of globe segment is multiplied by this. */
    expand,     /* TRUE while spring expands. */
    compress,   /* TRUE while sping compresses. */
    hold_ball,  /* TRUE while mage holds crystal ball. */
    ate_ball,   /* Prevents ball from being drawn if demon has eaten it. */ 
    it_bounced, /* If TRUE bounce sound is played. */
    it_crashed, /* If TRUE crash sounds are played. */
    play_game,  /* Flag for main game loop. */
    score,      /* Holds your current score. */
    last_score, /* Puts an asterisk beside your score if you make top six. */
    lost,       /* TRUE if the crystal ball is destroyed. */
    spin,       /* 0 through 5, chooses the correct background buffer
                   to create the spinning globe illusion.             */
    oops,       /* Makes sure that if you miss the ball you can't catch
                   it again.                                             */
    demon,      /* TRUE if a demon has appeared. */
    demon_pos,  /* Holds flag (LEFT or RIGHT) for demon position. */
    demon_ud;   /* Holds flag (UP or DOWN) for demon movement. */

/**************************************************************************/

/******************  The arrays for the globe and scores  *****************/

int current_globe[8][19],  /* Holds all the information needed for scoring
                              and drawing the blanked out sections of the
                              globe. Will change as game is played.       */
                              
    globex[19], /* holds x coordinates for drawing blanks & checking hits */
    globey[8],  /* holds y coordinates for drawing blanks & checking hits */
    
    top_six[6]; /* holds top six scores. */

/* This array is used to reset the globe back to its original state at
   the begining of each game. Each value represents a colored segment
   as defined earlier. This array will not change.                      */
   
int original_globe[8][19] = {{1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1},
                             {2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1,2},
                             {3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1,2,3},
                             {4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4},
                             {5,6,1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5},
                             {6,1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6},
                             {1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1},
                             {2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1,2}};

/**************************************************************************/

/**************************    Palette arrays    **************************/

int old_palette[16];  /* The color palette for the desktop. */

                      /* An all-black palette for when we want to hide
                         certain drawing processes.                    */
                         
int blank_palette[] = {0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000, 
                       0x000,0x000,0x000,0x000,0x000,0x000,0x000,0x000};

                     /* The color palette for the game. */

int new_palette[] = {0x000,0x700,0x070,0x770,0x457,0x707,0x477,0x544, 
                     0x655,0x522,0x353,0x653,0x336,0x535,0x346,0x777};
                     
/**************************************************************************/

/************************      Sprite arrays      *************************/

/* The sprites were cut using Peter Beery's Raster Sprite Editor (STlog #16)
   I then simply trimmed off any padding from the right side to allow me to
   use them with the assembly routines I developed for Ballbuster.        */

/* Data for blue crystal ball sprite. */

long ball[] = {
0x00000000,0x03C00000,
0x000003C0,0x0FF00000,
0x018007E0,0x1FF80180,
0x03C00FF0,0x1FF803C0,
0x06E00EF0,0x3EFC06E0,
0x03C00FF0,0x3FFC03C0,
0x018007E0,0x3FFC0180,
0x000023C4,0x3FFC2004,
0x00001008,0x1FF81008,
0x00001818,0x1FF81818,
0x00000E70,0x0FF00E70,
0x000003C0,0x03C003C0,
0x00000000,0x00000000};

/* Data for gold ball sprite. */

long gball[] = {
0x03C003C0,0x00000000,
0x0FF00FF0,0x00000000,
0x1FF81FF8,0x01800180,
0x1FF81FF8,0x03C003C0,
0x3EFC3EFC,0x06E006E0,
0x3FFC3FFC,0x07E007E0,
0x3FFC3FFC,0x03C003C0,
0x3FFC3FFC,0x01802184,
0x1FF81FF8,0x00001008,
0x1FF81FF8,0x00001818,
0x0FF00FF0,0x00000E70,
0x03C003C0,0x000003C0,
0x00000000,0x00000000};

/* Data for glass shards sprite. */

long shards[] = {
0x00000000,0x00000000,
0x00000000,0x00000000,
0x00000000,0x00000000,
0x00000000,0x00000000,
0x00000000,0x00000000,
0x00000000,0x00000000,
0x06000600,0x06000600,
0x00410241,0x03410041,
0x08002C90,0x2D900900,
0x00000021,0x10250004,
0x20002281,0x22992008,
0x54005452,0x54535401,
0x0000D004,0xD1060002,
0x00000900,0x1DD00450,
0x00040004,0x02040004,
0x00000000,0x00000000};

/* Data for explosion sprite. */

long explode[] = {
0x01020000,0x00000000,
0x09440100,0x00000000,
0xA02C0000,0x00000000,
0x76784048,0x00000000,
0x1FF31212,0x00000000,
0x1FF80740,0x02000200,
0x7FFC0DB0,0x00200020,
0x1FF907C1,0x00800080,
0x8EF006E0,0x02800280,
0x7FFA15C0,0x01C001C0,
0x0FF80350,0x02400240,
0x17F20180,0x00000000,
0x1BE90800,0x00000000,
0x34A00080,0x00000000,
0x01C80100,0x00000000,
0x82400000,0x00000000,
0x00000000,0x00000000};

/* Data for spring head sprite. */

long spring1[] = {
0x7FFE7FFE,0x47E227E4,
0x7FFE7FFE,0x43C223C4,
0x3FFC3FFC,0x21841188,
0x1FF81FF8,0x11880990,
0x0FF00FF0,0x0FF00000,
0x06600660,0x04200240,
0x06600660,0x02400240,
0x07E007E0,0x000005A0,
0x0A500A50,0x00000810,
0x15E815E8,0x04A01088,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x00000000,0x00000000};

/* Data for spring body sprite. */

long spring2[] = {
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x0E380E38,0x00000C18,
0x11C011C0,0x00801080,
0x00000000,0x00000000};

/* Data for mage-throwing-ball sprite. */

long mage1[] = {
0x00000000,0x00000200,0x00000000,0x00000040,
0x00000000,0x00000480,0x00000000,0x00000120,
0x00000000,0x00000500,0x00000000,0x000000A0,
0x00000000,0x00000540,0x00000000,0x000002A0,
0x02000200,0x02000180,0x00400040,0x00400180,
0x01000100,0x010000C0,0x00800080,0x00800300,
0x00800080,0x00800060,0x01000100,0x01000600,
0x00800080,0x00800060,0x01000100,0x01000600,
0x01200120,0x012000C3,0x44804480,0x44808300,
0x02400240,0x02400187,0x22402240,0x2240C180,
0x04800480,0x0480030F,0x11201120,0x1120E0C0,
0x09060906,0x09060619,0x70907090,0x70908860,
0x12141214,0x12160C2F,0xB848B848,0xF8486430,
0x24162416,0x2416182F,0xF824F824,0xF8246418,
0x48004800,0x4800301F,0x90129012,0x9012680C,
0x48004820,0x4800302F,0x90129412,0x9012640C,
0x44204420,0x44003807,0x24222422,0x2022C01C,
0x23002220,0x22001C26,0x20C42444,0x204444B8,
0x13801300,0x13000C03,0x41C840C8,0x40C88130,
0x1BC10B01,0x0B010400,0x83F880F0,0x80F00200,
0x1FFC0600,0x0600082F,0xFFF8C060,0xC0603C10,
0x3FFC0000,0x00001E87,0x7FFC4000,0x4000B178,
0x3FFC0000,0x00001C47,0x7FFE4000,0x4000B4BC,
0x7FFC0000,0x000033A7,0x7FFE4000,0x4000A0CC,
0x77DE0000,0x000024C7,0xFBEE8000,0x80006324,
0x6FFF0000,0x00000873,0xFFF60000,0x0000CC10,
0x5FFF0000,0x00001811,0xFFFA0000,0x00008818,
0x5FFF0000,0x00001C08,0xFFFA0000,0x00001038,
0x0FFF0003,0x00000E06,0xFFF0C000,0x00006070,
0x3FFF3800,0x00001F0E,0xFFFC001C,0x000070FC,
0x7FFD7C0C,0x000C1FD3,0xBFFE303E,0x3000CBF8,
0x7FF87E08,0x00080FF7,0x5FFE507E,0x5000AFF8,
0x7FFC7F8C,0x000C07F3,0x3FFE21FE,0x2000DFF0,
0x7FDE7FDE,0x001E41C1,0x03FE03FE,0x0000FBC2,
0x3F873F87,0x00073038,0x21FC21FC,0x2000DD0C,
0x0F800F80,0x00000E36,0x81F081F0,0x80006C70,
0x01C201C2,0x000201C8,0x43804380,0x40001380,
0x00000000,0x00000004,0x00000000,0x00002000,
0x00000000,0x00000000,0x00000000,0x00000000};

/* Data for mage-holding-ball sprite. */

long mage2[] = {
0x00000000,0x00000003,0x40004000,0x40008000,
0x00000000,0x00000007,0x20002000,0x2000C000,
0x00000000,0x0000000F,0x10001000,0x1000E000,
0x00060006,0x00060019,0x70007000,0x70008800,
0x00160016,0x0016002F,0xF800F800,0xF8006400,
0x00140014,0x0016002F,0xB800B800,0xF8006400,
0x00000000,0x0000001F,0x90009000,0x90006800,
0x00000020,0x0000002F,0x90009400,0x90006400,
0x00210021,0x00010005,0xA400A400,0xA0008000,
0x00000020,0x00000026,0x20002400,0x20004400,
0x00000000,0x00000003,0x40004000,0x40008000,
0x00010001,0x00010000,0x80008000,0x80000000,
0x00FC0000,0x00000003,0xFF00C000,0xC0003F00,
0x03FC0000,0x00000007,0x7FC04000,0x4000B9C0,
0x07FC0000,0x00000027,0x7FE04000,0x4000B400,
0x1FFC0000,0x00000747,0x7FF84000,0x4000A2E8,
0x3D8E0400,0x04000BF7,0xF1BC8020,0x80206FD0,
0x3C3F0400,0x04000BF3,0xFC3C0020,0x0020CFD0,
0x3F0F0300,0x03001CF1,0xF0FC00C0,0x00C08F38,
0x3FBF0080,0x00800F78,0xFDFC0100,0x01001EF0,
0x1F8F0003,0x00000FF6,0xF1F8C000,0x00006FF0,
0x3FFF2000,0x0000070E,0xFFFC0004,0x000070E4,
0x7FFD700C,0x000C1653,0xBFFE300E,0x3000CA6C,
0x7FF87008,0x000814F7,0x5FFE500E,0x5000AF2C,
0x7FFC718C,0x000C11F3,0x3FFE218E,0x2000DF88,
0x7FDE73DE,0x001E43C1,0x03FE03CE,0x0000FBCA,
0x3F873F87,0x000730B8,0x21FC21FC,0x2000DD0C,
0x0F800F80,0x00000E36,0x81F081F0,0x80006C70,
0x01C201C2,0x000201C8,0x43804380,0x40001380,
0x00000000,0x00000004,0x00000000,0x00002000,
0x00000000,0x00000000,0x00000000,0x00000000};

/* Data for gosh-how-did-I-miss-that sprite.*/

long mage3[] = {
0x00000000,0x00000000,0x00000000,0x00000100,
0x00000000,0x00000000,0x00000000,0x00001100,
0x00000000,0x00000000,0x00000000,0x00000920,
0x00000000,0x00000000,0x00000000,0x00002540,
0x00400040,0x0040003E,0x00000000,0x00001380,
0x00800080,0x00800071,0x00800080,0x00800F00,
0x01200120,0x012000CB,0x44804480,0x44808300,
0x02400240,0x02400187,0x22402240,0x2240C180,
0x04800480,0x0480030F,0x11201120,0x1120E0C0,
0x09060906,0x09060619,0x70907090,0x70908860,
0x12161216,0x12160C2F,0xF848F848,0xF8486430,
0x24142414,0x2416182F,0xB824B824,0xF8246418,
0x48064806,0x4806301F,0xF012F012,0xF012680C,
0x48004820,0x4800302F,0x90129412,0x9012640C,
0x44204420,0x44003806,0x24222422,0x2022401C,
0x23002220,0x22001C26,0x20C42444,0x204444B8,
0x13801300,0x13000C03,0x41C840C8,0x40C88130,
0x1BC10B01,0x0B010440,0x83F880F0,0x80F00200,
0x1FFC0600,0x0600082F,0xFFF8C060,0xC0603C10,
0x3FFC0000,0x00001E87,0x7FFC4000,0x4000B378,
0x3FFC0000,0x00001D07,0x7FFE4000,0x4000B43C,
0x7FFC0000,0x000033A7,0x7FFE4000,0x4000A0CC,
0x77DE0000,0x000024C7,0xFBEE8000,0x80006324,
0x6FFF0000,0x00000873,0xFFF60000,0x0000CC10,
0x5FFF0000,0x00001811,0xFFFA0000,0x00008818,
0x5FFF0000,0x00001C08,0xFFFA0000,0x00001038,
0x0FFF0003,0x00000E06,0xFFF0C000,0x00006070,
0x3FFF3800,0x00001F0E,0xFFFC001C,0x000070FC,
0x7FFD7C0C,0x000C1FD3,0xBFFE303E,0x3000CBF8,
0x7FF87E08,0x00080FF7,0x5FFE507E,0x5000AFF8,
0x7FFC7F8C,0x000C07F3,0x3FFE21FE,0x2000DFF0,
0x7FDE7FDE,0x001E41C1,0x03FE03FE,0x0000FBC2,
0x3F873F87,0x00073038,0x21FC21FC,0x2000DD0C,
0x0F800F80,0x00000E36,0x81F081F0,0x80006C70,
0x01C201C2,0x000201C8,0x43804380,0x40001380,
0x00000000,0x00000004,0x00000000,0x00002000,
0x00000000,0x00000000,0x00000000,0x00000000};

/* Data for left closed-mouth demon. */

long demonl1[] = {
0x00C00000,0x00C000C0,0x00000000,0x00000000,
0x00F80000,0x00F80078,0x00000000,0x00000000,
0x00FE0000,0x00FE004E,0x00000000,0x00000000,
0x00FF0000,0x00FF0053,0x00000000,0x00000000,
0x00FF0000,0x00FF0051,0xC0000000,0xC000C000,
0x00FF0000,0x00FF0005,0xF0000000,0xF0007000,
0x00FF0000,0x00FF0012,0xF8000000,0xF800B800,
0x00FF0000,0x00FF0044,0xFC000000,0xFC005C00,
0x00FF0300,0x00FF0355,0xFE000000,0xFE004E00,
0x00FF0700,0x00FF0790,0xFE000000,0xFE009E00,
0x007F0F80,0x007F0FD0,0xFC000000,0xFC00BC00,
0x003F1FC0,0x003F1FEB,0xF8000000,0xF800B800,
0x001F1FE0,0x001F18F2,0xF0000000,0xF0007000,
0x000F3FF0,0x000F3839,0xE0001000,0xE0007000,
0x00073FF8,0x0007339C,0xC0003800,0xC000E800,
0x03837C7C,0x000377CF,0x80007C00,0x8000DC00,
0x03C17C3E,0x0001666F,0x0400F800,0x0000BC00,
0x03E07C1F,0x00006633,0x0C00F000,0x00007C00,
0x03F07C4F,0x0000671A,0x1C00E800,0x0000B400,
0x01F87E07,0x0000638D,0x3C00C000,0x00006400,
0x00F87F07,0x000061CC,0x3C00C000,0x00006C00,
0x00787F87,0x000070FD,0x3800C400,0x00007C00,
0x00007FFF,0x0000547F,0x0000FC00,0x0000E400,
0x00007FEF,0x00004E23,0x0000DC00,0x00009C00,
0x00007FF7,0x00004FD0,0x0000B800,0x00002800,
0x00007FFF,0x0000418B,0x0000F800,0x00005800,
0x00007FFF,0x000065CC,0x0000F800,0x0000D800,
0x00007FFF,0x00007B98,0x0000F800,0x00007800,
0x00017FFD,0x000076A1,0x00007C00,0x00001C00,
0x00087FF8,0x00007D48,0x40005C00,0x00004C00,
0x00007FC7,0x00007900,0x0000CC00,0x00000400,
0x00007FFF,0x00007983,0x0000FC00,0x00008400,
0x00003FFF,0x000028CE,0x0000FC00,0x0000EC00,
0x00003FFF,0x00003C68,0x0000FC00,0x00002C00,
0x00001FFF,0x00001E00,0x0000F800,0x00001800,
0x00001FFF,0x00001D02,0x0000F800,0x0000B800,
0x00000FFF,0x00000FC0,0x0000F000,0x00007000,
0x000003FF,0x000003C5,0x0000E000,0x0000E000,
0x000000FF,0x000000FF,0x00008000,0x00008000,
0x00000000,0x00000000,0x00000000,0x00000000};

/* Data for left open-mouth demon. */

long demonl2[] = {
0x00600000,0x00600060,0x00000000,0x00000000,
0x007C0000,0x007C003C,0x00000000,0x00000000,
0x007F0000,0x007F0027,0x00000000,0x00000000,
0x007F0000,0x007F0029,0x80000000,0x80008000,
0x007F0000,0x007F0028,0xE0000000,0xE000E000,
0x007F0000,0x007F0002,0xF8000000,0xF800B800,
0x007F0000,0x007F0009,0xFC000000,0xFC005C00,
0x007F0000,0x007F0062,0xFE000000,0xFE002E00,
0x007F0000,0x007F002A,0xFF000000,0xFF00A700,
0x007F0180,0x007F01C8,0xFF000000,0xFF004F00,
0x003F03C0,0x003F03E8,0xFE000000,0xFE005E00,
0x001F07E0,0x001F07F5,0xFC000000,0xFC00DC00,
0x000F0FF0,0x000F0CF9,0xF8000000,0xF8003800,
0x00071FF8,0x00071D8C,0xF0000C00,0xF000B400,
0x01E31E1C,0x00031BE6,0xE4001800,0xE0006C00,
0x01F13E0E,0x00013B17,0xCE003000,0xC000DA00,
0x01F83E07,0x00003308,0x9E006000,0x8000B200,
0x01FC7E23,0x00007704,0x3E00C800,0x00006200,
0x00FC7F03,0x00006586,0x3E00C000,0x0000E200,
0x007C7F83,0x000064C6,0x3C00C200,0x0000E600,
0x003C7FC3,0x00006C7E,0x3800C600,0x0000BE00,
0x00007FFF,0x0000663F,0x0000FE00,0x0000E200,
0x00007FEF,0x00007383,0x0000F600,0x0000C200,
0x00007FF7,0x00007A84,0x0000EC00,0x0000A000,
0x00007FFF,0x00005CDF,0x0000FC00,0x0000F800,
0x00007FFF,0x00004E78,0x0000FC00,0x00000C00,
0x00037FFF,0x00034C20,0xF000FC00,0xF0000400,
0x001F7FFF,0x001D4443,0xB800BC00,0x9800B000,
0x003D7FFD,0x00344D9D,0x90009400,0x90009000,
0x00047FC4,0x00004904,0x80008400,0x00008000,
0x00007F80,0x00006A00,0x00000400,0x00000000,
0x00007F00,0x00006800,0x00000400,0x00000000,
0x001D7F00,0x0000781D,0xC0000400,0x0000C000,
0x003F7F00,0x00007832,0xE0000C00,0x00006000,
0x003F3F80,0x00002A31,0xE0000800,0x00002000,
0x001F3F80,0x00003E19,0xC0000800,0x00004000,
0x000F1FE0,0x00001B88,0xC0001800,0x00004000,
0x00071FF0,0x00001CC4,0xC0001000,0x00004000,
0x00030FF8,0x00000C62,0xC0001000,0x0000C000,
0x00010FFC,0x00000E71,0x80003000,0x00008000,
0x000007FF,0x00000718,0x0000E000,0x00000000,
0x000003FF,0x000003E7,0x0000C000,0x0000C000,
0x000001FF,0x000001FF,0x00008000,0x00008000,
0x0000007E,0x0000007E,0x00000000,0x00000000,
0x00000000,0x00000000,0x00000000,0x00000000};

/* Data for right closed-mouth demon. */

long demonr1[] = {
0x00060000,0x00060006,0x00000000,0x00000000,
0x003E0000,0x003E003C,0x00000000,0x00000000,
0x00FE0000,0x00FE00E4,0x00000000,0x00000000,
0x01FE0000,0x01FE0194,0x00000000,0x00000000,
0x07FE0000,0x07FE0714,0x00000000,0x00000000,
0x1FFE0000,0x1FFE1D40,0x00000000,0x00000000,
0x3FFE0000,0x3FFE3A90,0x00000000,0x00000000,
0x7FFE0000,0x7FFE7444,0x00000000,0x00000000,
0xFFFE0001,0xFFFEE555,0x00008000,0x00008000,
0xFFFE0001,0xFFFEF213,0x0000C000,0x0000C000,
0x7FFC0003,0x7FFC7A17,0x0000E000,0x0000E000,
0x3FF80007,0x3FF83BAF,0x0000F000,0x0000F000,
0x1FF0000F,0x1FF01C9E,0x0000F000,0x00003000,
0x0FE0101F,0x0FE01D38,0x0000F800,0x00003800,
0x07C0383F,0x07C02E73,0x0000F800,0x00009800,
0x03837C7C,0x038077E7,0x80007C00,0x0000DC00,
0x41073EF8,0x01007BEC,0x80007C00,0x0000CC00,
0x600F1FF0,0x00007D98,0x80007C00,0x0000CC00,
0x701F2FE4,0x00005AB1,0x80007C00,0x0000CC00,
0x783F07C0,0x00004D63,0x0000FC00,0x00008C00,
0x783E07C1,0x00006C67,0x0000FC00,0x00000C00,
0x383C47C3,0x00007D7E,0x0000FC00,0x00001C00,
0x00007FFF,0x00004FFC,0x0000FC00,0x00005400,
0x000077EF,0x00007388,0x0000FC00,0x0000E400,
0x00003BDF,0x00002817,0x0000FC00,0x0000E400,
0x00003FFF,0x000035A3,0x0000FC00,0x00000400,
0x00003FFF,0x00003667,0x0000FC00,0x00004C00,
0x00003FFF,0x00003C33,0x0000FC00,0x0000BC00,
0x01007D7F,0x0000710A,0x0000FC00,0x0000DC00,
0x0420743F,0x00006425,0x0000FC00,0x00007C00,
0x000067C7,0x00004001,0x0000FC00,0x00003C00,
0x00007FFF,0x00004383,0x0000FC00,0x00003C00,
0x00007FFF,0x00006EE6,0x0000F800,0x00002800,
0x00007FFF,0x0000682C,0x0000F800,0x00007800,
0x00003FFF,0x00003000,0x0000F000,0x0000F000,
0x00003FFF,0x00003A81,0x0000F000,0x00007000,
0x00001FFF,0x00001C07,0x0000E000,0x0000E000,
0x00000FFF,0x00000F47,0x00008000,0x00008000,
0x000003FE,0x000003FE,0x00000000,0x00000000,
0x00000000,0x00000000,0x00000000,0x00000000};

/* Data for right open-mouth demon. */

long demonr2[] = {
0x00060000,0x00060006,0x00000000,0x00000000,
0x003E0000,0x003E003C,0x00000000,0x00000000,
0x00FE0000,0x00FE00E4,0x00000000,0x00000000,
0x01FE0000,0x01FE0194,0x00000000,0x00000000,
0x07FE0000,0x07FE0714,0x00000000,0x00000000,
0x1FFE0000,0x1FFE1D40,0x00000000,0x00000000,
0x3FFE0000,0x3FFE3A90,0x00000000,0x00000000,
0x7FFE0000,0x7FFE7446,0x00000000,0x00000000,
0xFFFE0000,0xFFFEE554,0x00000000,0x00000000,
0xFFFE0001,0xFFFEF213,0x00008000,0x00008000,
0x7FFC0003,0x7FFC7A17,0x0000C000,0x0000C000,
0x3FF80007,0x3FF83BAF,0x0000E000,0x0000E000,
0x1FF0000F,0x1FF01C9F,0x0000F000,0x00003000,
0x0FE0301F,0x0FE02D31,0x0000F800,0x0000B800,
0x27C71838,0x07C03667,0x80007800,0x0000D800,
0x738F0C70,0x03805BE8,0x80007C00,0x0000DC00,
0x791F06E0,0x01004D10,0x80007C00,0x0000CC00,
0x7C3F13C4,0x00004620,0x80007E00,0x0000EE00,
0x7C3F03C0,0x00004761,0x0000FE00,0x0000A600,
0x3C3E43C1,0x00006763,0x0000FE00,0x00002600,
0x1C3C63C3,0x00007D7E,0x0000FE00,0x00003600,
0x00007FFF,0x000047FC,0x0000FE00,0x00006600,
0x00006FF7,0x000043C1,0x0000FE00,0x0000CE00,
0x000037EF,0x00000521,0x0000FE00,0x00005E00,
0x00003FFF,0x00001FFB,0x0000FE00,0x00003A00,
0x00003FFF,0x0000301E,0x0000FE00,0x00007200,
0x0FC03FFF,0x0FC02004,0x0000FE00,0x00003200,
0x1DF83DFF,0x19B80DC2,0x0000FE00,0x00002200,
0x09BC29BF,0x092C09B9,0x0000FE00,0x0000B200,
0x01202123,0x00000120,0x0000FE00,0x00009200,
0x00002001,0x00000000,0x0000FE00,0x00005600,
0x00002000,0x00000000,0x0000FE00,0x00001600,
0x03B82000,0x000003B8,0x0000FE00,0x00001E00,
0x07FC3000,0x0000064C,0x0000FE00,0x00001E00,
0x07FC1001,0x0000048C,0x0000FC00,0x00005400,
0x03F81001,0x00000298,0x0000FC00,0x00007C00,
0x03F01807,0x00000211,0x0000F800,0x0000D800,
0x03E0080F,0x00000223,0x0000F800,0x00003800,
0x03C0081F,0x00000346,0x0000F000,0x00003000,
0x01800C3F,0x0000018E,0x0000F000,0x00007000,
0x000007FF,0x00000018,0x0000E000,0x0000E000,
0x000003FF,0x000003E7,0x0000C000,0x0000C000,
0x000001FF,0x000001FF,0x00008000,0x00008000,
0x0000007E,0x0000007E,0x00000000,0x00000000,
0x00000000,0x00000000,0x00000000,0x00000000};

long blank = Malloc(140L);   /* Holds the blank segment sprite. */
long blue = Malloc(272L);    /* Holds the blue segment sprite. */
long green = Malloc(272L);   /* Holds the green segment sprite. */
long red = Malloc(272L);     /* Holds the red segment sprite. */
long lblue = Malloc(272L);   /* Holds the light blue segment sprite. */
long violet = Malloc(272L);  /* Holds the violet segment sprite. */
long yellow = Malloc(272L);  /* Holds the yellow segment sprite. */

/* The following arrays will hold up to 16 copies of the namesake sprite,
   each one shifted the correct number of pixels to the right.            */

long lball[16];
long lgball[16];
long lshards[16];
long lexplode[16];
long lspring1[3];
long lspring2[3];
long lmage1[16];
long lmage2[16];
long lmage3[16];
long ldemonl1[4];
long ldemonl2[3];
long ldemonr1[8];
long ldemonr2[8];

long lblank[16];

long globe_cut = Malloc(24000L); /* A "cookie-cutter" for the globe. */
long globe[6];                   /* Array for the backgrounds. */

/**************************************************************************/

/***************************************************************************/
/* The following data are for the sounds used in Ballbuster.  These sounds */
/* were created using the G.I.S.T. Sound Editor from Antic's The Catalog.  */
/* The G.I.S.T. link file, SNDSUBS.O, is required to create an executeable */
/* file.                                                                   */
/***************************************************************************/

extern          snd_on(),snd_off(),stop_snd(),get_prior();
extern          install_int(),remove_int();
extern          init_snds();

int bounce[] =
{
 8, 1250, -1, 15, 1, 3, -16384, -1,
 -8192, 0, 0, -1, -8937, 0, 0, 0,
 0, 0, 1, -4057, 2971, -4057, 2971, 338,
 5214, 0, 0, 4056, -2971, 1537, -10240, -1118,
 572, 1537, -10240, -1118, 572, -1118, 572, 0,
 1, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0
};

int bang[] =
{
 200, 477, 31, 15, 1, 15, 0, -1,
 -1267, 10, 3277, -1, -2573, 0, 0, 0,
 0, 0, 1, -9, 18470, -140, -32161, 0,
 5504, 28, 31652, 0, 3679, 1656, 13223, 430,
 -25398, 430, -25398, -1180, -16857, -307, 25764, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0
};

int chomp[] =
{
 60, 83, 18, 15, 1, 15, 0, -1,
 -15124, 0, 0, -15, 0, 5, 6554, 5,
 6554, 0, 1, -4080, -256, -4080, -256, 81,
 -9174, 28665, 0, -28665, 0, 28665, 0, -1720,
 6554, 13759, 13107, -3584, -8192, -1720, 6554, 0,
 1, 0, 0, 0, 0, -1, -3694, -31,
 0, 31, 0, 15, -32768, -5, 2621, 0
};

int gulp[] =
{
 48, 625, -1, 15, 1, 3, -16384, -1,
 -8192, 0, 0, -1, -8937, 0, 0, 0,
 0, 0, 1, -4057, 2971, -4057, 2971, 338,
 5214, 0, 0, 4056, -2971, 28665, 0, -8,
 -10895, 57, 21627, -3584, -8192, -8, -10895, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0
};

int glass[] =
{
 40, 125, 0, 15, 1, 15, 0, 0,
 0, 15, 0, -1, -6342, 7, -32768, 4,
 -32768, 0, 1, 337, -3251, 21966, -14732, -25039,
 -1652, -3072, -16384, 3071, 16384, 28665, 0, -2294,
 -13107, 18345, -26214, -3584, -8192, -2294, -13107, 0,
 1, -31, 0, -31, 0, 0, 27862, 13,
 -23593, -14, 23593, 15, -32768, -14, 23593, 0
};

int bell[] =
{
 20, 119, -1, 15, 1, 3, -16384, -1,
 -3488, 6, -16384, -1, -2458, 0, 0, 0,
 0, 0, 1, -631, -26608, -2522, 24639, 1260,
 -12319, 0, 0, 2521, -24639, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0
};

int squeek[] =
{
 40, 625, -1, 15, 1, 3, -16384, -1,
 -8192, 0, 0, -1, -8937, 0, 0, 0,
 0, 0, 1, -1931, 16019, -1931, 16019, 29,
 -19403, 0, 0, 1930, -16019, 21966, -14732, -125,
 -16803, 790, -12851, -3452, 28424, -125, -16803, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0
};

/**************************************************************************/

/****************************    The buffers    ***************************/

long SCREEN1, SCREEN2, HIDDEN, SHOWN, DESK;

/**************************************************************************/



/* The main program function. */

main()
{
     register int rez;
     
     play_game = PLAY; /* Set play_game flag to PLAY. */
     
     last_score = 1;   /* Set last_score to 1 so you don't see asterisks 
                          next to all the zeros when you first start the
                          game.                                           */
                          
     appl_init();   /* Standard call to initialize program. */
     
     install_int(); /* Install and initialize the */
     init_snds();   /* G.I.S.T. sound driver.     */
     
     open_vwork();  /* Open the workstation. */
     
     rez = Getrez(); /* Get the current resolution. */
     
     if(rez != 0)  /* If you ran the game in anything but low resolution. */
     
         form_alert(1, "[1][Ballbuster must be run|in low resolution!][Oops!]");
         
     else  /* If the resolution is low, let's play Ballbuster! */
     {
         init_game();        /* Go and build the game. */
         print_top_scores(); /* Draw the title screen and scores. */
     
         while(play_game == PLAY) /* While you want to play. */
         {
             set_up_game();   /* Set up for a new game. */
             play_the_game(); /* Go and play a game.  Have fun! */
         }
         close_up(); /* Free up the memory. */
     }
    
    v_clsvwk(handle); /* Close the workstation. */
    remove_int();     /* Remove the G.I.S.T. sound driver. */
    appl_exit();      /* So long! */
}

/* Game initialization function. */

init_game()
{
     register int i, row, col, x = 104, y = 24;
     
     MOUSE_OFF; /* Turn off the mouse, we won't need it for this game. */
     
     v_clrwk(handle);  /* Clear the screen. */
     
     for(i=0; i<16; old_palette[i++]=Setcolor(i, -1)); /* Save old palette */
     
     Setpalette(blank_palette); /* Set palette to all black to hide all
                                   the stuff we will do.                */
     
     /* Set up two buffers for the screen. One we can make changes to
        while we display the other.                                   */
        
     SCREEN1 = ((Malloc(32768L) & 0xffffff00) + 0x0100);
     SCREEN2 = ((Malloc(32768L) & 0xffffff00) + 0x0100);    
     
     SCREEN1 = Physbase(); /* SCREEN1 now points to the screen we see.  */
     
     HIDDEN = SCREEN2;     /* HIDDEN points to the buffer we draw on. */   
     SHOWN = SCREEN1;      /* SHOWN points to the buffer we see.      */
     DESK = Physbase();    /* DESK will always point
                              to the physical screen. */

     for(i=0; i<6; globe[i++] = Malloc(32000L)); /* Set up 6 buffers for
                                                    the background.       */
                                                    
     for(i=0; i<16; lblank[i++] = Malloc(280L)); /* Set up an array for 16
                                                    shifted blank sprites.*/
                                                    
   /* Initialize the current_globe array and the globex & globey arrays. */
       
     for(row = 0; row < 8; row++)
     {
         for(col = 0; col < 19; col++)
         {
             globex[col] = x;
             current_globe[row][col] = original_globe[row][col];
             x = x + 16;
         }
         x = 104; globey[row] = y; y = y + 16;
     }
     
     make_segment(); /* Go create the segments of the globe. */
     make_globe();   /* Draw the six background buffers. */
     
   /* Allocate the memory needed for all the shifted sprite arrays. The 
      amount to allocate is twice the size of the original sprite.      */
      
     for(i=0; i<16; i++)
     {
         lball[i] = Malloc(208L);
         lgball[i] = Malloc(208L);
         lshards[i] = Malloc(256L);
         lexplode[i] = Malloc(256L);
         lmage1[i] = Malloc(1248L);
         lmage2[i] = Malloc(992L);
         lmage3[i] = Malloc(1184L);
     }
     
     ldemonl1[2] = Malloc(1280L);
     ldemonl1[3] = Malloc(1280L);
     ldemonl2[2] = Malloc(1440L);
     ldemonr1[7] = Malloc(1280L);
     ldemonr2[6] = Malloc(1440L);
     ldemonr2[5] = Malloc(1440L);
     
     lspring1[2] = Malloc(256L);
     lspring2[2] = Malloc(256L);
          
    /* Now go and create the shifted sprite arrays. */

     for(i=0; i<16; i++)
     {
        shift_sprite(ball, lball[i], i, sprite_w, ball_h);
        shift_sprite(gball, lgball[i], i, sprite_w, ball_h);
        shift_sprite(shards, lshards[i], i, sprite_w, sprite_h);
        shift_sprite(explode, lexplode[i], i, sprite_w, sprite_h);
        shift_sprite(mage1, lmage1[i], i, mage_w, mage1_h);
        shift_sprite(mage2, lmage2[i], i, mage_w, mage2_h);
        shift_sprite(mage3, lmage3[i], i, mage_w, mage3_h);
     }

     shift_sprite(demonl1, ldemonl1[2], 2, demon_w, demon1_h);
     shift_sprite(demonl1, ldemonl1[3], 3, demon_w, demon1_h);
     shift_sprite(demonl2, ldemonl2[2], 2, demon_w, demon2_h);
     shift_sprite(demonr1, ldemonr1[7], 7, demon_w, demon1_h);
     shift_sprite(demonr2, ldemonr2[6], 6, demon_w, demon2_h);
     shift_sprite(demonr2, ldemonr2[5], 5, demon_w, demon2_h);
    
     shift_sprite(spring1, lspring1[2], 2, sprite_w, sprite_h);
     shift_sprite(spring2, lspring2[2], 2, sprite_w, sprite_h);
    
   /* Since we don't need the original sprites anymore we
      will free up some of that precious memory.          */
     
     Mfree(demonr2);
     Mfree(demonr1);
     Mfree(demonl2);
     Mfree(demonl1);
     Mfree(mage3);
     Mfree(mage2);
     Mfree(mage1);
     Mfree(spring2);
     Mfree(spring1);
     Mfree(explode);
     Mfree(shards);
     Mfree(gball);
     Mfree(ball);

     for(i=0; i<6; i++) /* Draw bottom of spring on each background. */
     {
         plot_sprite(1,lspring2,globe[i],sprite_w,sprite_h,2,181);
         
         top_six[i] = 0; /* Might as well initialize the top scores
                            array while we are at it.               */
     }
}

/* This function simply sets the variables, flags and arrays to the
   values needed to start a new game.                               */
   
set_up_game()
{
     register int row, col;

     ball_y = 180;
     ball_trak = UP; 
     b_speed = 0;
     n_balls = 3; 
     bonus_ball = 0; 
     gold_ball = FALSE;
     hold_ball = TRUE;
     oops = FALSE;
     lost = FALSE;
     score = 0; 
     spin = 0;
     demon = FALSE; 
     ate_ball = FALSE;
     it_bounced = FALSE; 
     it_crashed = FALSE;
     top_of_spring = 39;
     expand = FALSE; 
     compress = FALSE;

     for(row = 0; row < 8; row++)
        for(col = 0; col < 19; col++)
             current_globe[row][col] = original_globe[row][col];
}

/* Open the workstation.  Standard stuff. */

open_vwork()
{
     register int i;
     handle = graf_handle(&dummy,&dummy,&dummy,&dummy);
     for(i=0;i<10;work_in[i++]=1);
     work_in[10] = 2;
     v_opnvwk(work_in,&handle,work_out);
}

/* This is the main game loop. */

play_the_game()
{
    v_clrwk(handle);
    Setpalette(new_palette);
    
    while(n_balls > 0)     /* While you still have crystal balls left. */
    {
        spin_globe();   /* Go and "spin" the globe. */
        check_demon();  /* Go and conjure up a demon, or remove one. */
        move_mage();  /* Move the mage where you want it. */
        
        if(lost == TRUE)   /* If you've lost the ball, do this... */
            lost_ball();
        else
            move_ball();   /* ...otherwise move the ball. */
            
        check_border();  /* See if the ball has hit one of the borders. */
        check_gobble();  /* See if the ball has touched a demon. */
        check_spring();  /* See if the ball is on the spring. */
        draw_scene();    /* Puts together the graphic elements. */
        check_globe_hit(); /* See if you hit a globe segment and if so,
                              do the explosions, adjust score, etc.     */
        print_score(); /* Print the mana you've released and the number
                          of crystal balls left at the top of the screen. */
    }
    
    draw_scene();   /* Draw the screen one last time. */
    print_score();  /* Print the stuff at the top. */
    game_over();    /* Print NO BALLS! in flashing lights and wait
                       for you to press a mouse button.             */
    check_top_scores();  /* Routine to see if you made the top six
                            scores and put you in there if you did. */
    print_top_scores();  /* Back to the title screen. */
}

/* This function simply prints the NO BALLS! message in various colors
   while it waits for you to press a mouse button.                     */

game_over()
{
    int m_button, m_x, m_y, color = 1;

    vswr_mode(handle, 2);
    vst_effects(handle, 5);
    vst_height(handle, 28, &dummy, &dummy, &dummy, &dummy);
    
    m_button = FALSE;
    while(m_button == FALSE)
    {
        vq_mouse(handle, &m_button, &m_x, &m_y);
        if(color < 7)
            color++;
        else
            color = 1;
        vst_color(handle, color);
        v_gtext(handle, 91, 100, "NO BALLS!");
    }
    vst_height(handle, 6, &dummy, &dummy, &dummy, &dummy);
    vst_effects(handle, 0);
    init_snds();            /* Turn off all sounds. */
}

/* Here the function resets the old colors and frees up the memory so we
   can go back to the dektop without having to reset the ST.  This is an
   especially nice feature for us hard disk owners!                      */

close_up()
{
    register int i;
 
    Setscreen(DESK, DESK, -1);       /* Set the screen to DESK. */
    v_clrwk(handle);                 /* Clear the screen. */
    Setpalette(old_palette);         /* Restore the desktop palette. */
    
    Mfree(lspring2[2]);     /* Free up the memory used */
    Mfree(lspring1[2]);     /* in the sprite arrays.   */
    
    Mfree(ldemonr2[5]);
    Mfree(ldemonr2[6]);
    Mfree(ldemonr1[7]);
    Mfree(ldemonl2[2]);
    Mfree(ldemonl1[3]);
    Mfree(ldemonl1[2]);
        
    for(i=15; i>=0; i--)
    {
        Mfree(lmage3[i]);
        Mfree(lmage2[i]);
        Mfree(lmage1[i]);
        Mfree(lexplode[i]);
        Mfree(lshards[i]);
        Mfree(lgball[i]);
        Mfree(lball[i]);
    }
    
    for(i=15; i>=0; Mfree(lblank[i--])); /* Free up the blank array. */
    
    for(i=5; i>=0; Mfree(globe[i--])); /* Free up the memory used in the 
                                          six background buffers.         */

    Mfree(SCREEN2);  /* Free memory used in the   */
    Mfree(SCREEN1);  /* SHOWN and HIDDEN buffers. */
}

/* A simple sorting routine to place your score into the proper slot in
   the top six scores array and slide the remaining scores down one,
   dropping the lowest score out... if you made it, that is!            */
   
check_top_scores()
{
    register int i, temp_score;
    
    last_score = score;
    
    for(i = 0; i < 6; i++)
        if(score > top_six[i])
        {
            temp_score = top_six[i];
            top_six[i] = score;
            score = temp_score;
        }
}

/* This function prints the mana you've released (your score) and the
   number of crystal balls remaining to you at the top of the screen. 
   It also looks to see if you have won a bonus ball.                  */
   
print_score()
{
    register int i;
    register char the_score[10], balls_left[2];
    
    if(bonus_ball >= 5000) /* If the bonus counter reaches 5000 points... */
    {
        n_balls++;                      /* give you an extra ball...      */
        bonus_ball = bonus_ball - 5000; /* reset the bonus counter...     */
        snd_on(bell,-1,15,-1,5);        /* and ring a bell.               */
    }
    
    for(i = 0; i < 10; the_score[i++] = '\0'); /* Clear the_score array. */
    
    sprintf(the_score, "%d", score);    /* Convert score into a string. */
    sprintf(balls_left, "%d", n_balls); /* Convert n_balls into a string. */
    
    vswr_mode(handle, 1);                /* Set writing mode to replace.  */
    vst_color(handle, 1);                /* Set text color to white.      */
    v_gtext(handle, 40, 7, the_score);   /* Print the score next to MANA. */
    
    if(n_balls >= 3)                /* If you have three or more balls... */
        vst_color(handle, 3);       /* set the color to green.            */
    else
    if(n_balls == 2)               /* If there are only two balls left...*/
        vst_color(handle, 6);      /* set the color to yellow.           */
    else
        vst_color(handle, 2);       /* Otherwise set the color to red. */
    
    v_gtext(handle, 311, 7, balls_left); /* Print number of balls left. */
}

/* Here the function to draw the title screen is called and the top six
   scores are printed onto it in various colors.  If the score from the
   last game played made it to the top six, an asterisk will be displayed
   next to that score.                                                    */
   
print_top_scores()
{
    register int i, j, text_y = 80;
    register char a_score[10];

    Setpalette(blank_palette);
    draw_title_screen();
    
    for(i = 0; i < 6; i++)
    {
        for(j=0; j < 10; a_score[j++] = '\0')
        
        sprintf(a_score, "%d", top_six[i]);
    
        vst_color(handle, i + 2);
        v_gtext(handle, 139, text_y, a_score);
        
        if(top_six[i] == last_score)
        {
            vst_color(handle, 1);
            v_gtext(handle, 187, text_y, "*");
            last_score = -1;
        }
        text_y = text_y + 15;
    }
    
    Setpalette(new_palette);
    play_again();
}

/* This is the function to draw the title screen.  Most of it simply draws
   three rounded rectangles in light blue, medium blue and dark blue for a
   nice border and prints text in various sizes, effects and colors. After
   this is done it places some sprites around for a decorative touch.     */
   
draw_title_screen()
{
    int  rect[4], colors[3];
    register int i;

    colors[0] = 5; colors[1] = 4; colors[2] = 12;
    
    v_clrwk(handle);
    
    for(i = 0; i < 3; i++)
    {
        rect[0] = 0 + i; rect[1] = 0 + i;
        rect[2] = 319 - i; rect[3] = 199 - i;
    
        vsl_color(handle, colors[i]);
        v_rbox(handle, rect);
    }
    
    vst_effects(handle, 25);
    vst_height(handle, 28, &dummy, &dummy, &dummy, &dummy);
    
    vst_color(handle, 2);
    v_gtext(handle, 70, 30, "Ballbuster");
    
    vst_height(handle, 6, &dummy, &dummy, &dummy, &dummy);

    vst_effects(handle, 5);
    vst_color(handle, 12);
    v_gtext(handle, 107, 41, "By Eric Scott");
    
    vst_effects(handle, 0);
    vswr_mode(handle, 2);

    vst_color(handle, 10);
    v_gtext(handle, 44, 60, "Top six scores for Ballbuster");
    
    vst_color(handle, 6);
    v_gtext(handle, 43, 59, "Top six scores for Ballbuster");
    
    vst_color(handle, 15);
    v_gtext(handle, 36, 181, "Click left mouse button to play");
    
    vst_color(handle, 7);
    v_gtext(handle, 35, 180, "Click left mouse button to play");
    
    vst_color(handle, 8);
    v_gtext(handle, 32, 191, "Click right mouse button to quit");
    
    vst_color(handle, 6);
    v_gtext(handle, 31, 190, "Click right mouse button to quit");

    plot_sprite(1,ldemonl1,SHOWN,demon_w,demon1_h,3,4);
    plot_sprite(1,ldemonr2,SHOWN,demon_w,demon2_h,293,4);
    
    plot_sprite(1,lball,SHOWN,sprite_w,ball_h,61,72);
    plot_sprite(1,lmage1,SHOWN,mage_w,mage1_h,53,120);
    plot_sprite(1,lexplode,SHOWN,sprite_w,sprite_h,242,70);
    plot_sprite(1,lball,SHOWN,sprite_w,ball_h,242,72);
    plot_sprite(1,lmage3,SHOWN,mage_w,mage3_h,234,124);
}

/* This function waits for you to press the left or right mouse buttons,
   which determine whether you will play a game or quit Ballbuster.       */
   
play_again()
{
    register int button;
    
    play_game = FALSE;
    
    while(play_game == FALSE)
        vq_mouse(handle, &play_game, &dummy, &dummy);
        
    if(play_game != PLAY)
    {
        MOUSE_ON;
        button = form_alert(2, "[2][|Do you really|want to quit?][Play|Quit]");
        play_game = button++;
        MOUSE_OFF;
    }
}

/* This function shifts the sprite "shift" number of pixels right.  The
   shifted sprite is placed into the array pointed to by "list".        */

shift_sprite(sprite, array, shift, width, height)
long sprite, array;
int shift, width, height;
{
    asm
    {
        move.w D6, -(A7)       /* Save register D6. */

        move.w width(A6), D5   /* D5 is the width. */
        divu   #8, D5          /* Divide width by 4. */

        moveq #0,  D2          /* moveq #0 clears data registers  */
        moveq #0,  D3          /* faster than clr.l               */

        move.l sprite(A6), A0  /* A0 now points to the sprite. */
        move.l array(A6), A1   /* A1 now points to the array. */
        
        move.w shift(A6), D0   /* D0 now holds the amount to shift. */
        
        move.w height(A6), D2  /* D2 now holds the sprite height. */
        
    loop:
        
        move.w D5, D3   /* D3 is the width. */
        
    shift_it:
    
        moveq  #0, D1     /* Clear D1. */
        moveq  #0, D4     /* Clear D4. */
        moveq  #0, D6

        move.w (A0)+, D4  /* D4 now holds a word of the sprite. */
        swap   D4         /* Move it to the high word. */
        lsr.l  D0, D4     /* Shift it right the amount in D0. */
        move.w D4, D1     /* D1 now holds high word of sprite. */
        swap   D4         /* Move it back to low word. */
        move.w D4, (A1)+  /* Move low word into array. */

        moveq  #0, D4     /* Clear D4. */
        move.w (A0)+, D4  /* D4 now holds a word of the sprite. */
        swap   D4         /* Move it to the high word. */
        lsr.l  D0, D4     /* Shift it right the amount in D0. */
        move.w D4, D6     /* D6 now holds high word of sprite. */
        swap   D4         /* Move it back to low word. */
        move.w D4, (A1)+  /* Move low word into array. */
        move.w D1, (A1)+  /* Move high word into array. */
        move.w D6, (A1)+  /* Move high word into array. */
        
    dbf D3, shift_it      /* Repeat shift for width/4 times. */ 
    
    dbf D2, loop    /* Repeat loop for each line of sprite. */

    move.w (A7)+, D6  /* Restore register D6. */
    }
}

/* This fast assembly routine will draw a shifted sprite or erase a globe
   segment.  The correct sprite to plot is determined by ANDing the x
   coordinate.                                                            */

plot_sprite(mode,array,buffer,width,height,xpos,ypos)
long *array, buffer;
int mode, width, height, xpos, ypos;
{
    asm
    {
        move.w D5, -(A7)  /* Save contents of D5. */

        move.w #160, D4       /* Calculate the */
        move.w width(A6), D0  /* amount to add */
        ext.l  D0             /* to the buffer */
        divu   #2, D0         /* for each scan */
        sub.w  D0, D4         /* line of the   */
        ext.l  D4             /* sprite. Put the long result in D4. */
        
        move.w width(A6), D5  /* Adjust width */
        divu   #16, D5        /* for the OR   */
        subq.w #1, D5         /* loop. Put it in D5. */
        
        move.w ypos(A6), D1    /* D1 is the y coordinate. */
        mulu   #160, D1        /* Multiply D1 by bytes per screen line 
                                  to get vertical offset.              */
        add.l D1, buffer(A6)   /* Add the vertical offset to buffer addr. */
         
        move.w xpos(A6), D0    /* D0 is the x coordinate. */
        andi.w #0xfffff0, D0   /* Mask off the lower 4 bits. */
        asr.l #1, D0           /* divide D0 by 2. */
        add.l D0, buffer(A6)   /* Add horizontal offset to buffer addr. */
        
        move.l array(A6), A1   /* Move sprite array into A1. */
        move.w xpos(A6), D0    /* D0 is the x coordinate. */
        andi.w #15, D0         /* AND D0 to get the array index. */
        add.w  D0, D0          /* Shift D0 left */
        add.w  D0, D0          /* two bits.     */
        move.l 0(A1, D0), A0   /* A0 now points to the shifted sprite. */
        move.l buffer(A6), A1  /* A1 now points to low word of buffer. */
        move.l A1, A2          
        addq.l #8, A2          /* A2 now points to high word of buffer. */

        cmpi.w #1, mode(A6)    /* Is this a sprite? */
        bne    do_blank        /* If not, erase a segment. */

        move.w height(A6), D3  /* D3 has value of sprite height. */

    lines:
    
        move.w  D5, D2  /* D2 is the # of 16 horizontal pixels */
                        /* for this scan line of the sprite.   */
    plot:

        move.l (A0)+, D0 /* Move low word of the sprite into D0. */
        move.l (A0)+, D1 /* Move high word of sprite into D1. */
        or.l D0, (A1)+ /* OR low word of sprite with low word of buffer. */
        or.l D1, (A2)+ /* OR high word of sprite with high word of buffer.*/
        
        move.l (A0)+, D0  /* Next 8  */
        move.l (A0)+, D1  /* pixels. */
        or.l D0, (A1)+   
        or.l D1, (A2)+   
        
    dbf D2, plot          /* Do next 16 pixels of sprite if needed. */
        
        adda.l D4, A1  /* Go to next scan line. */
        adda.l D4, A2  /* Go to next scan line. */
        
    dbf D3, lines  /* Repeat until all lines of sprite have been plotted. */
    bra end        /* We are done with this sprite! */
    
    do_blank:            /* Erase a globe segment. */

        move.w height(A6), D2   /* D2 is the height. */
        
        move.l (A0)+, D0  /* The first word. */
        move.l (A0)+, D1  /* The second word. */
        not.l  D0
        not.l  D1

    b_lines:
        
        and.l D0, (A1)+  /* AND low word with low word of buffer. */
        and.l D1, (A2)+  /* AND high word with high word of buffer. */
        
        and.l D0, (A1)+  /* etc... */
        and.l D1, (A2)+
        
        adda.l D4, A1 /* Next scan line. */
        adda.l D4, A2 /* Next scan line. */
        
    dbf D2, b_lines /* Do for 16 lines. */

    end:
        move.w (A7)+, D5  /* Restore D5. */
    }
}

/* This is a fast assembly routine to copy the contents of the source 
   graphics buffer into the destination graphics buffer.              */
   
copy_buffer(source, dest)
register long *source, *dest;
{
    asm
    {
        move.w #1000, D0

    loop:
        move.l   (source)+, (dest)+  /* Move 32 bytes of   */
        move.l   (source)+, (dest)+  /* source buffer into */
        move.l   (source)+, (dest)+  /* the destination    */
        move.l   (source)+, (dest)+  /* buffer.            */
        move.l   (source)+, (dest)+  
        move.l   (source)+, (dest)+  
        move.l   (source)+, (dest)+  
        move.l   (source)+, (dest)+
    dbf D0, loop                     /* 1000 times. */  
    }
}

/* This function flips the HIDDEN and SHOWN buffers.  Usually all work on
   a screen is done on the HIDDEN buffer and then "flipped" with the
   SHOWN buffer, achieving nice, smooth animation.                        */
   
flip_buffers()
{
    asm
    {
        move.w  #0x25, -(A7)  /* Wait for next   */
        trap    #14           /* Vertical blank. */

        move.w #-1, -(A7)         /* Show                           */
        move.l HIDDEN(A4), -(A7)  /* the                            */
        move.l HIDDEN(A4), -(A7)  /* HIDDEN                         */
        move.w #5, -(A7)          /* buffer,                        */
        jsr    xbios(PC)          /* actually                       */
        adda   #12, A7            /* Setscreen(HIDDEN, HIDDEN, -1). */

        move.l HIDDEN(A4), D0        /* Switch the addresses */
        move.l SHOWN(A4), HIDDEN(A4) /* of the SHOWN and the */
        move.l D0, SHOWN(A4)         /* HIDDEN buffers.      */
    }
}

/* Here we have the old, familiar C function to put sprites on the screen.
   Easy to use, but oh! so slow!                                          */

stamp_sprite(sprite,mode,width,height,dx1,dy1)
long sprite;
int mode, width, height, dx1, dy1;
{
     FDB s_m,    /* Form definition block for source */
         scr_m;  /* Form definition block for screen */
     int pxy[8];  /* Coordinates for source and destination rectangles */
     
     s_m.fd_addr = sprite;  /* Store address of sprite data in FDB */
     s_m.fd_w = width;             /* Store width of sprite in FDB */
     s_m.fd_h = height;            /* Store height of sprite in FDB */
     s_m.fd_wdwidth = width/16;    /* Store sprite width/16 in FDB */
     s_m.fd_stand = 0;             /* Set up for RC */
     s_m.fd_nplanes = 4;           /* Low resolution (4 bit planes) */

     scr_m.fd_addr = 0;     /* Put it on the screen. */

     pxy[0] = 0;          /* Upper-left X coordinate of source block      */
     pxy[1] = 0;          /* Upper-left Y coordinate of source block      */
     pxy[2] = width;      /* Lower-right X coordinate of source block     */
     pxy[3] = height;     /* Lower-right Y coordinate of source block     */
     pxy[4] = dx1;        /* Upper-left X coordinate of destination block */
     pxy[5] = dy1;        /* Upper-left Y coordinate of destination block */
     pxy[6] = dx1 + width;     /* Lower-right X coordinate of dest. block */
     pxy[7] = dy1 + height;    /* Lower-right Y coordinate of dest. block */
     
     vro_cpyfm(handle,mode,pxy,&s_m,&scr_m); /* Do raster operation */
}

/* This function will draw an 8 by 8 circle on the screen made up of the
   colored segments created in build_segment().                          */

draw_globe()
{
    register int globe_x = 103, globe_y = 24;
    register int row, col;
    for(row = 0; row < 8; row++)
    {
        for(col = 0; col < 8; col++)
        {
            switch(current_globe[row][col]) /* Determine which segment to */
            {                               /* draw from current_globe.   */
                case BLUE:
                stamp_sprite(blue,7,32,sprite_h,globe_x,globe_y);
                break;
    
                case GREEN:
                stamp_sprite(green,7,32,sprite_h,globe_x,globe_y);
                break;
    
                case RED:
                stamp_sprite(red,7,32,sprite_h,globe_x,globe_y);
                break;
    
                case LBLUE:
                stamp_sprite(lblue,7,32,sprite_h,globe_x,globe_y);
                break;
    
                case VIOLET:
                stamp_sprite(violet,7,32,sprite_h,globe_x,globe_y);
                break;
    
                case YELLOW:
                stamp_sprite(yellow,7,32,sprite_h,globe_x,globe_y);
                break;
            }
            globe_x = globe_x + 16;  /* Next x coordinate. */
        }
        globe_x = 103;           /* Reset x coordinate. */
        globe_y = globe_y + 16;  /* Next y coordinate. */
    }

    /* Now turn that big square into a big circle! */

    stamp_sprite(globe_cut,1,globe_w,globe_h,80,18);
}

/* This function "rotates" the current_globe array one segment to the
   left and updates spin for the background buffer.                   */
 
spin_globe()
{
    register int first_column[8][1];
    register int row, col, color;
    
  /* Save the values for the left side of the globe before they
     "slip off the edge". */

    for(row = 0; row < 8; row++) 
        first_column[row][0] = current_globe[row][0];
        
  /* Now move all the values one segment left. */

    for(row = 0; row < 8; row++)
        for(col = 0; col < 19; col++)
        {
           if(col + 1 < 18)  /* If we have not reached the last column... */

           /* Slide segment left. */ 
              current_globe[row][col] = current_globe[row][col+1];

           /* Otherwise re-introduce the first segment. */
           else                              
              current_globe[row][col] = first_column[row][0];
        }
        
    spin++;                 /* Increment spin. */
    if(spin == 6) spin = 0; /* Reset spin. */
}

/* This function checks the current_globe array for BLANKs made by
   knocking out segments and erases those segments.                */
         
globe_hits()
{
    register int col, row;

    for(row = 0; row < 8; row++)
        for(col = 0; col < 8; col++)
            if(current_globe[row][col] == BLANK)
                plot_sprite(0,lblank,HIDDEN,sprite_w,sprite_h,globex[col],globey[row]);
}

/* This function draws the spring on the left side of the screen. */

draw_spring()
{
    register int y;
        
    plot_sprite(1,lspring1,HIDDEN,sprite_w,sprite_h,2,top_of_spring);
    
    y = top_of_spring + 16;
    
    while(y < 180)
    {
        plot_sprite(1,lspring2,HIDDEN,sprite_w,sprite_h,2,y);
        y = y + 16;
    }
}

/* This function draws most of the graphic elements in the game.  It
   also controls the demon animation and plays certain sounds.    */

draw_scene()
{
    copy_buffer(globe[spin], HIDDEN); /* Move the correct background into
                                         the HIDDEN buffer.               */

    globe_hits();   /* Erase any globe segments that are knocked out. */
    draw_spring();  /* Draw the golden spring. */

 /*************************************************************************/
 /* This section will draw the correct mage at the proper x-y location. */

    if(hold_ball == TRUE)
        plot_sprite(1,lmage2,HIDDEN,mage_w,mage2_h,mage_x,mage2_y);
    else
    if(lost == TRUE)
        plot_sprite(1,lmage3,HIDDEN,mage_w,mage3_h,mage_x,mage3_y);
    else
        plot_sprite(1,lmage1,HIDDEN,mage_w,mage1_h,mage_x,mage1_y);

 /*************************************************************************/

 /*************************************************************************/
 /* If the demon has caught your ball, this section handles the munching  */
 /* animation and appropriate sounds.                                     */

    if(demon > TRUE)
    {
        if(demon == 2 || demon == 4 || demon == 6 || demon == 8)
        {
            if(demon_pos == LEFT)
                plot_sprite(1,ldemonl2,HIDDEN,demon_w,demon2_h,18,demon_y);
            else
                plot_sprite(1,ldemonr2,HIDDEN,demon_w,demon2_h,294,demon_y);
            
            if(demon_pos == LEFT)
                ball_x = 26;
            else
                ball_x = 295;
                
            ball_y = demon_y + 24;
            
            if(demon < 5)
                plot_sprite(1,lshards,HIDDEN,sprite_w,sprite_h,ball_x,ball_y);
                
            snd_on(chomp,0,15,-1,5);
        }
        
        else
        if(demon == 3 || demon == 5 || demon == 7 || demon == 9)
        {
            if(demon_pos == LEFT)
                plot_sprite(1,ldemonl1,HIDDEN,demon_w,demon1_h,18,demon_y);
                
            else
                plot_sprite(1,ldemonr1,HIDDEN,demon_w,demon1_h,295,demon_y);
                
            snd_on(chomp,1,15,-1,5);
        }
        
        else
            demon = FALSE;
        
        demon++;
        
        if(demon == 10)
            snd_on(gulp,0,15,-1,5);
    }
 /*************************************************************************/

 /*************************************************************************/
 /* This section moves demon up or down and draws the demon on the right  */
 /* or left side of the screen.                                           */

    if(demon == TRUE)
    {
       if(demon_ud == UP && demon_y > 12)     /* Slide demon up if it  */
           demon_y--;                         /* is not at top.        */
       else
       
       if(demon_ud == DOWN && demon_y < 126)  /* Side demon down if it */
           demon_y++;                         /* is not at bottom.     */
           
       if(demon_pos == LEFT)
           plot_sprite(1,ldemonl1,HIDDEN,demon_w,demon1_h,18,demon_y);
           
       else
           plot_sprite(1,ldemonr1,HIDDEN,demon_w,demon1_h,295,demon_y);
    }
 /*************************************************************************/

 /*************************************************************************/
 /* This section draws the crystal ball at the correct position and plays */
 /* the bounce or crashing glass sounds when needed.                      */

    if(ball_y > 186) ball_y = 186;
    
    if(lost == FALSE && gold_ball == FALSE)
        plot_sprite(1,lball,HIDDEN,sprite_w,ball_h,ball_x,ball_y);

    else
    if(lost == FALSE && gold_ball > FALSE)
        plot_sprite(1,lgball,HIDDEN,sprite_w,ball_h,ball_x,ball_y);

    else
    if(ate_ball == FALSE)
        plot_sprite(1,lshards,HIDDEN,sprite_w,sprite_h,ball_x,ball_y);
       
    if(it_bounced == TRUE)
        snd_on(bounce,0,15,-1,5);
        
    if(it_crashed == TRUE)
    {
        plot_sprite(1,lexplode,HIDDEN,sprite_w,sprite_h,ball_x,ball_y);
        snd_on(bang,0,15,-1,5);
        snd_on(glass,1,15,-1,5);
    }
 /*************************************************************************/

    flip_buffers(); /* Everythings drawn, now we display it. */
}

/* This function does all ball movement. */

move_ball()
{
    if(hold_ball == TRUE)  /* If the mage is holding the ball. */
    {
        ball_y = 180;           /* Position the crystal ball */
        ball_x = mage_x + 8;  /* in his lap.               */
    }

    else
    if(expand == FALSE && compress == FALSE)  /* If the ball is not
                                                 on the spring.     */
        switch(ball_trak)
        {
            case UP:  /* When the ball is going up. */
            
            ball_y = ball_y - b_speed;  /* Move the ball up. */
            
            if(b_speed > 0)  /* If the ball is still moving...*/
                b_speed--;   /* Decrease of speed due to gravity. */
            else
                ball_trak = DOWN;  /* Gravity pulls the ball down. */
            break;
        

            case DOWN:  /* When the ball is going down. */
                
            ball_y = ball_y + b_speed;  /* Move the ball down. */
            
            if(b_speed < 16)  /* If the ball hasn't reached max speed... */
                b_speed++;    /* Increase of speed due to gravity. */
            break;
            

            case LEFT:  /* When the ball is going left. */
            
            ball_x = ball_x - b_speed; /* Move the ball left. */
            ball_y++;  /* Ball drops slightly due to gravity. */
            
            if(b_speed < 2+max_speed)/* If ball hasn't reached max speed... */
                b_speed++;           /* Increase of speed due to gravity. */
            else
                b_speed = 2+max_speed;
            break;
            

            case RIGHT:  /* When the ball is going right. */
            
            ball_x = ball_x + b_speed;  /* Move the ball right. */
            ball_y++;  /* Ball drops slightly due to gravity. */
            
            if(b_speed < 2+max_speed)/* If ball hasn't reached max speed... */
                b_speed++;           /* Increase of speed due to gravity. */
            else
                b_speed = 2+max_speed;
            break;
            

            case UAL:  /* When the ball is going up and to the left. */
            
            ball_y = ball_y - b_speed;  /* Move the ball up. */
            ball_x = ball_x - b_speed;  /* Move the ball left. */
            
            if(b_speed > 0)  /* If the ball is still moving... */
                b_speed--;   /* Decrease of speed due to gravity. */
            else
                ball_trak = DAL; /* Gravity pulls the ball down. */
            
            break;
            
            case UAR:  /* When the ball is going up and to the right. */
            
            ball_y = ball_y - b_speed;  /* Move the ball up. */
            ball_x = ball_x + b_speed;  /* Move the ball right. */
            
            if(b_speed > 0)  /* If the ball is still moving... */
                b_speed--;   /* Decrease of speed due to gravity. */
            else
                ball_trak = DAR; /* Gravity pulls the ball down. */
            
            break;
            
            case DAL:  /* When the ball is going down and to the left. */
                
            ball_y = ball_y + b_speed;  /* Move the ball down. */
            ball_x = ball_x - b_speed;  /* Move the ball left. */
            
            if(b_speed < max_speed)/* If ball hasn't reached max speed... */
                b_speed++;         /* Increase of speed due to gravity. */
            else
                b_speed = max_speed;
            break;
        
            case DAR:  /* When the ball is going down and to the right. */
            
            ball_y = ball_y + b_speed;  /* Move the ball down. */
            ball_x = ball_x + b_speed;  /* Move the ball right. */
            
            if(b_speed < max_speed)/* If ball hasn't reached max speed... */
                b_speed++;         /* Increase of speed due to gravity. */
            else
                b_speed = max_speed;
            break;
        }
    
    it_bounced = FALSE;  /* Reset it_bounced. */
}

/* When the crystal ball has either crashed or been eaten, this function
   will see if you press the left mouse button and then reset some flags. */

lost_ball()
{
    int m_button = FALSE, m_x, m_y;
    
    it_crashed = FALSE;  /* Reset these flags to stop the */
    it_bounced = FALSE;  /* bounce and crash sounds.      */
    
    if(demon < MUNCH)    /* You may have to wait for the */
    {                    /* demon to finish his snack.   */

        vq_mouse(handle, &m_button, &m_x, &m_y);  /* Did you press
                                                     a button?     */
    
        if(m_button == TRUE)  /* Did you press the left mouse button? */
        {
            lost = FALSE;      /* Reset  */
            oops = FALSE;      /* some   */
            hold_ball = TRUE;  /* of     */
            ate_ball = FALSE;  /* the    */
            gold_ball = FALSE; /* flags. */

            init_snds();              /* Turn off all sounds. */
            snd_on(squeek,0,15,-1,5); /* The mage makes a happy noise. */

            ball_y = 180;          /* Make sure the ball is */
            ball_x = mage_x + 8; /* in the mages lap.   */
        }
    }
}
    
/* This function is called when the ball has reached line 157 on the
   screen.  It checks to see if the mages head or hands have touched
   the center of the ball.                                             */

check_mage()
{
    register int sweet_spot, left_hand, mage_noggin, right_hand;
    
    sweet_spot = ball_x + 8;    /* The center of the crystal ball. */
    left_hand = mage_x + 3;   /* The left hand of the mage. */
    mage_noggin = mage_x + 16; /* The mages noggin. */
    right_hand = mage_x + 29; /* The right hand of the mage. */
    
 /* Did you catch the crystal ball? */

    if((sweet_spot >= left_hand) && (sweet_spot <= right_hand) && oops == FALSE)
    {
        if(ball_trak != UP)     /* Set the bounce sound */
            it_bounced = TRUE;  /* flag to TRUE.        */
            
        if(sweet_spot >= left_hand && sweet_spot < mage_noggin - 4)

            ball_trak = UAR;  /* If it hit the left hand,
                                 send the ball up and right. */
        else
        if(sweet_spot > mage_noggin + 4 && sweet_spot <= right_hand)

            ball_trak = UAL;  /* If it hit the right hand,
                                 send the ball up and left. */

        else
            ball_trak = UP;   /* Otherwise it must have hit his 
                                 noggin, so send it straight up! */

        if(b_speed < 16) b_speed++; /* Give the ball a little push. */

        ball_y = 157; /* Make sure ball will be drawn at the right place. */
    }
    
    else  /* Uh oh... you dropped the crystal ball! */
    {
        oops = TRUE;  /* Set this flag so you can't "recover the fumble". */

        if(ball_y >= 186 && lost == FALSE)   /* When the ball hits the */
        {                                    /* bottom of the screen.  */
            n_balls--;         /* Lose one ball. */
            lost = TRUE;       /* Yep, its gone. */
            it_crashed = TRUE; /* Set flag for the smashing glass sound. */
        }
    }
}

/* This function will randomly generate a demon somewhere on the left or
   right side of the screen.  If a demon is already present it may be
   removed.  No more than one demon will be on screen at any time.       */

check_demon()
{
    register int a_demon;
    
    a_demon = (int) Random();          /* Make a_demon a random number */
    a_demon = abs(a_demon) % 100 + 1;  /* between 1 and 100.           */
    
    if(demon == TRUE)      /* If there is a demon present already, */
    {
        if(a_demon > 98)   /* see if it is time to remove it.      */
            demon = FALSE;
        else

            if(a_demon < 5)
                demon_ud = DOWN; /* Slide the demon down. */
            else

            if(a_demon > 93)
                demon_ud = UP;   /* Slide the demon up. */
    }
    else
    
    if(a_demon < 8 && demon == FALSE)  /* Otherwise, conjure one up! */
    {
        demon = TRUE;                      /* Yes, there is now a demon. */

        demon_y = (int) Random();          /* Put it somewhere between    */
        demon_y = abs(demon_y) % 114 + 12; /* the top and bottom borders. */

        demon_pos = (int) Random();         /* Decide whether to place  */
        demon_pos = abs(demon_pos) % 2 + 1; /* it on the left or right  */
        demon_pos = demon_pos + 2;          /* side of the screen.      */
    }
}

/* This function checks to see if the crystal ball has hit the demon
   somewhere on his ugly puss.                                       */

check_gobble()
{
 /* If there is a demon present and the ball isn't a golden mana ball,
    go and check to see if it's lunchtime.                             */

    if(demon == TRUE && ate_ball == FALSE && gold_ball == FALSE)
    {
        if(demon_pos == LEFT && ball_x > 16)  
        {
            if(ball_x < 31 && ball_y >= demon_y &&  /* Is it */
               ball_y <= demon_y + 35)              /* food? */
            {
                demon = MUNCH;   /* Then pig out! */
                n_balls--;       /* Lose a ball.  */
                lost = TRUE;     /* Last you'll see of that ball! */
                ate_ball = TRUE; /* Of course he did! */
            }
        }
        
        else
            if(ball_x > 284 && ball_y >= demon_y    /* Is it */
               && ball_y <= demon_y + 35)           /* food? */
            {
                demon = MUNCH;   /* Then pig out! */
                n_balls--;       /* Lose a ball.  */
                lost = TRUE;     /* Last you'll see of that ball! */
                ate_ball = TRUE; /* Of course he did! */
            }
    }
}

/* This function checks to see if the crystal ball has hit one of the
   borders or has entered the slot above the spring.                  */

check_border()
{
  /* Did it enter the slot above the spring? */
    
    if(ball_x < 17 && ball_y < 31 && compress == FALSE && expand == FALSE)
    {
        ball_x = 2;        /* Line it up with the spring. */
        it_bounced = TRUE; /* Set the flag for the bounce sound. */
        compress = TRUE;  /* Set the flag for the spring routine. */
    }
    else

  /* Did it hit the left border? */

    if(ball_x < 17  && compress == FALSE && expand == FALSE)
    {
        ball_x = 17;       /* Line it up with the left border. */
        change_trak();     /* Change the direction of the ball. */
        it_bounced = TRUE; /* Set the flag for the bounce sound. */
    }
    else

    if(ball_x > 303)       /* Did it hit the right border? */
    {
        ball_x = 303;      /* Line it up with the right border. */
        change_trak();     /* Change the direction of the ball. */
        it_bounced = TRUE; /* Set the flag for the bounce sound. */
    }
    
    if(ball_y < 12)        /* Did it hit the top border? */
    {
        ball_y = 12;       /* Line it up with the top border. */
        change_trak();     /* change the direction of the ball. */
        it_bounced = TRUE; /* Set the flag for the bounce sound. */
    }
    else
    
  /* If it is near the bottom we have to see if the mage caught it. */
  
    if(ball_y >= 157 && ball_x >= 17 && hold_ball == FALSE)
        check_mage();
        
    if(oops == TRUE && ball_y <= 157) /* Just in case you got */
        oops = FALSE;                 /* a lucky bounce...    */
}

/* This funtion does all the action if the crystal ball is on the spring. */
   
check_spring()
{
    register int which_way;
    
    if(expand == TRUE)      /* If the spring is expanding. */
    {
        if(b_speed < 16)    /* If the speed is under the max speed... */
            b_speed++;      /* increase the speed.                    */
        
        ball_y = ball_y - b_speed;  /* Move the ball up. */
        
        if(top_of_spring > 39)            /* If the spring is   */
        {                                 /* still expanding... */
            top_of_spring = ball_y + 12;  /* move the spring up and */
            snd_on(gulp,1,15,-1,5);       /* make the strange sound. */
        }
        
        else
        {
            top_of_spring = 39;  /* Otherwise, the spring has expanded. */
            ball_y = 12;         /* Position the ball */
            ball_x = 17;         /* outside the slot. */
            expand = FALSE;      /* Reset the flag. */

            which_way = (int) Random();           /* Decide where */
            which_way = abs(which_way) % 2 + 1;   /* the golden   */
            if(which_way == 1) ball_trak = RIGHT; /* ball will    */
            else ball_trak = DAR;                 /* head.        */
            it_bounced = TRUE;   /* Set the flag for the bounce sound. */
        }
    }
    
    if(compress == TRUE)  /* If the spring is compressing. */
    {
        if(top_of_spring > 180)  /* If the spring is fully compressed. */
        {
            top_of_spring = 180; /* Position the spring head */
            ball_y = 166;        /* and the ball.            */
            compress = FALSE;    /* End this phase. */
            expand = TRUE;       /* Set the flag for the next phase. */
            b_speed = 0;         /* Make sure the ball is stopped. */
            gold_ball++;         /* Supercharge the crystal ball! */
        }
        
        else                     /* Squeeze the spring some more. */
        {
            ball_y = ball_y + 2;             /* Move the ball down. */
            if(ball_y > top_of_spring - 12)  /* If the ball is */
            {                                /* on the spring. */
                top_of_spring = ball_y + 12; /* Squeeze the spring. */
                snd_on(glass,1,15,-1,5);     /* Make a strange sound. */
            }
        }
    }
}

/* This function will change the present direction the ball is heading
   after it has hit a border.                                          */
   
change_trak()
{
    register int which_way;
    
    which_way = (int) Random();
    which_way = abs(which_way) % 3 + 1;
    
    switch(ball_trak)
    {
        case UP:
        if(which_way == 1) ball_trak = DAL;
            
        else
        if(which_way == 2) ball_trak = DAR;
        
        else
        ball_trak = DOWN;
        
        break;
        
        case DOWN:
        if(which_way == 1 && ball_x > 15) ball_trak = UAL;
            
        else
        if(which_way == 2 && ball_x < 303) ball_trak = UAR;
        
        else
        ball_trak = UP;
        
        break;
        
        case LEFT:
        if(which_way == 1 && ball_y > 12) ball_trak = UAR;
            
        else
        if(which_way == 2 && ball_y < 186) ball_trak = DAR;
        
        else
        ball_trak = RIGHT;
        
        break;
        
        case RIGHT:
        if(which_way == 1 && ball_y > 12) ball_trak = UAL;
            
        else
        if(which_way == 2 && ball_y < 186) ball_trak = DAL;
        
        else
        ball_trak = LEFT;
        
        break;
        
        case UAL:
        if(which_way == 1 && (ball_x < 303 && ball_y > 12)) ball_trak = UAR;
            
        else
        if(which_way == 2 && (ball_x > 17 && ball_y < 186)) ball_trak = DAL;
        
        else
        if(ball_y > 12) ball_trak = UP;
        
        else
            ball_trak = DOWN;
        
        break;
        
        case UAR:
        if(which_way == 1 && (ball_x > 17 && ball_y > 12)) ball_trak = UAL;
            
        else
        if(which_way == 2 && (ball_x < 303 && ball_y < 186)) ball_trak = DAR;
        
        else
        if(ball_y > 12) ball_trak = UP;
        
        else
            ball_trak = DOWN;
        
        break;
        
        case DAL:
        if(which_way == 1 && (ball_x < 303 && ball_y < 186)) ball_trak = DAR;
        
        else
        if(which_way == 2 && (ball_x > 17 && ball_y > 12)) ball_trak = UAL;
        
        else
            ball_trak = DOWN;
        
        break;
        
        case DAR:
        if(which_way == 1 && (ball_x > 17 && ball_y < 186)) ball_trak = DAL;
        
        else
        if(which_way == 2 && (ball_x < 303 && ball_y > 12)) ball_trak = UAR;
        
        else
            ball_trak = DOWN;
        
        break;
    }
}

/* This function moves the mage along the bottom of the screen.  The
   mage moves toward the x coordinate of the mouse.                   */
   
move_mage()
{
    int m_button = FALSE, m_y, old_x;
    
    old_x = mage_x;  /* Save the current x coordinate of the mage. */
    
    vq_mouse(handle, &m_button, &mage_x, &m_y); /* Get the new x posi-
                                                     tion from the mouse. */
                                                     
    if(m_button == BOTH && hold_ball == TRUE)/* This will make the mage */
    {                                        /* throw the ball if he is   */
        hold_ball = FALSE;                   /* holding it and you press  */
        b_speed = 4; ball_trak = UP;         /* both mouse buttons.       */
        snd_on(squeek,0,15,-1,5);      /* The mage makes a happy noise. */
    }
    
    if(mage_x < old_x - max_mage)   /* This limits the speed at which */
        mage_x = old_x - max_mage;  /* the mage can move left.      */
    
    if(mage_x > old_x + max_mage)   /* This limits the speed at which */
        mage_x = old_x + max_mage;  /* the mage can move right.     */
        
    if(mage_x < 18)    /* This keeps the mage from   */
        mage_x = 18;   /* moving past the left border. */
        
    if(mage_x > 286)   /* This keeps the mage from    */
        mage_x = 286;  /* moving past the right border. */
}

/* This function determines whether the crystal ball has hit a segment,
   and if so, adjusts the score and does a little explosion routine.    */
   
check_globe_hit()
{
    register int row, col, ballx_center, bally_center, top_speed, hit = FALSE;
    int mana, pixel;
    
    ballx_center = ball_x + 7;  /* Get the coordinates of the tiny black  */
    bally_center = ball_y + 4;  /* hole on the ball so we can find out    */
                                /* if it hit something.                   */
                                
    v_get_pixel(handle, ballx_center, bally_center, &pixel, &mana);

    if(pixel > 0)               /* If the hole was anything but black... */
    {
       for(row = 0; row < 8; row++)    /* Check and see if the ball */
          for(col = 0; col < 8; col++) /* is over any segments.     */
          {
              if((ballx_center >= globex[col] && bally_center >= globey[row]
                 && ballx_center <= globex[col] + 16 && bally_center <= globey[row] + 16)
                 && hit == FALSE)
              {
                  mana = current_globe[row][col];  /* Get the mana
                                                      in the segment. */
                  adjust_score(mana);              /* Add to score. */
                  current_globe[row][col] = BLANK; /* The segments gone. */
                  
                  ball_trak = (int) Random();             /* Change direc-*/
                  ball_trak = abs(ball_trak) % 8 + 1;     /* tion of ball.*/
                  b_speed = (int) Random();               /* Change the   */
                  b_speed = abs(b_speed) % 16 + 1;        /* ball speed.  */
                  
                  if(ball_trak > 5 && b_speed > max_speed)
                      b_speed = max_speed;
                  
                  hit = TRUE;                  /* Avoid doing this again. */
                  
                  snd_on(bang,0,15,-1,5);  /* Do explosion! */
                  plot_sprite(0,lblank,SHOWN,sprite_w,sprite_h,globex[col],globey[row]);
                  plot_sprite(1,lexplode,SHOWN,sprite_w,sprite_h,globex[col],globey[row]);
                  plot_sprite(1,lball,SHOWN,sprite_w,ball_h,ball_x,ball_y);
              }
         }
    }
}

/* This function will take the segment you hit and award you the amount
   of mana released.  If you have the golden ball the amount will be
   doubled, or more!                                                      */

adjust_score(segment)
int segment;
{
    switch(segment)
    {
        case RED:
        score = score + 500 * (gold_ball + 1);
        bonus_ball = bonus_ball + 500 * (gold_ball + 1);
        break;
        
        case BLUE:
        score = score + 10 * (gold_ball + 1);
        bonus_ball = bonus_ball + 10 * (gold_ball + 1);
        break;
        
        case GREEN:
        score = score + 25 * (gold_ball + 1);
        bonus_ball = bonus_ball + 25 * (gold_ball + 1);
        break;
        
        case LBLUE:
        score = score + 50 * (gold_ball + 1);
        bonus_ball = bonus_ball + 50 * (gold_ball + 1);
        break;
        
        case VIOLET:
        score = score + 75 * (gold_ball + 1);
        bonus_ball = bonus_ball + 75 * (gold_ball + 1);
        break;
        
        case YELLOW:
        score = score + 100 * (gold_ball + 1);
        bonus_ball = bonus_ball + 100 * (gold_ball + 1);
        break;
    }
}


/* This function oversees the creation of all the globe segments. */

make_segments()
{
    register int i;

    build_segment(1,1,0,15);
    cut_sprite(blank, SHOWN, sprite_w, sprite_h);
    v_clrwk(handle);
    build_segment(4,12,1,16);
    cut_sprite(blue, SHOWN, 32, sprite_h);
    build_segment(2,10,1,16);
    cut_sprite(red, SHOWN, 32, sprite_h);
    build_segment(3,11,1,16);
    cut_sprite(green, SHOWN, 32, sprite_h);
    build_segment(6,14,1,16);
    cut_sprite(yellow, SHOWN, 32, sprite_h);
    build_segment(7,15,1,16);
    cut_sprite(violet, SHOWN, 32, sprite_h);
    build_segment(5,13,1,16);
    cut_sprite(lblue, SHOWN, 32, sprite_h);

   /* Make 16 shifted copies of the blank sprite. */
   
    for(i=0; i<16; i++)
        shift_sprite(blank, lblank[i], i, sprite_w, sprite_h);
}

/* This function first makes a "cookie-cutter" sprite that is used to
   mask out the corners of the globe.  It then goes on to draw the six
   background buffers.                                                  */

make_globe()
{
    register int i;
    
    v_clrwk(handle);
    vsf_color(handle, 1);
    v_circle(handle, 88, 70, 63);
    cut_sprite(globe_cut, SHOWN, globe_w, globe_h);

    for(i=0; i<6; i++)
    {
        v_clrwk(handle); 
        draw_screen();
        draw_globe();
        copy_buffer(SHOWN, globe[i]);
        spin_globe();
    }
    v_clrwk(handle);

    Mfree(globe_cut);  /* Free up some of the   */
    Mfree(yellow);     /* memory because we     */
    Mfree(violet);     /* don't need this stuff */
    Mfree(lblue);      /* anymore.              */
    Mfree(red);
    Mfree(green);
    Mfree(blue);
    Mfree(blank);
}

/* This function draws the borders and writes the text on the background
   buffers.                                                              */

draw_screen()
{
    int rect[16], color[3];
    register int i;
    
    color[0] = 2; color[1] = 10;

    rect[0] = 0; rect[1] = 9;
    rect[2] = 319; rect[3] = 9;
    rect[4] = 319; rect[5] = 199;
    rect[6] = 0; rect[7] = 199;
    rect[8] = 0; rect[9] = 9;

    vsl_color(handle, 7);
    v_pline(handle, 5, rect);
    
    for(i = 0; i < 2; i++)
    {
        rect[0] = 1 + i; rect[1] = 10 + i;
        rect[2] = 318 - i; rect[3] = 10 + i;
        rect[4] = 318 - i; rect[5] = 198 - i;
        rect[6] = 18 - i; rect[7] = 198 - i;
        rect[8] = 18 - i; rect[9] = 35;
        rect[10] = 18 - i; rect[11] = 198 - i;
        rect[12] = 1 + i; rect[13] = 198 - i;
        rect[14] = 1 + i; rect[15] = 10 + i;
    
        vsl_color(handle, color[i]);
        v_pline(handle, 8, rect);
    }
    
    vswr_mode(handle, 2); vst_color(handle, 11);
    v_gtext(handle, 1, 7, "MANA:");
    vst_color(handle, 3);
    v_gtext(handle, 0, 6, "MANA:");

    vst_effects(handle, 17);
    vst_color(handle, 2);
    v_gtext(handle, 119, 7, "Ballbuster");
    
    vst_effects(handle, 0);
    vst_color(handle, 12);
    v_gtext(handle, 264, 7, "BALLS:");
    vst_color(handle, 5);
    v_gtext(handle, 263, 6, "BALLS:");
}

/* This function draws the colored globe segments, really just filled
   rectangles with a design made from 3 boxes in a darker color.      */

build_segment(lcolor, dcolor, b_x, e_x)
int lcolor, dcolor, b_x, e_x;
{
    int segment[4], detail[4];
    register int i;
    
    segment[0] = b_x; segment[1] = 0;
    segment[2] = e_x; segment[3] = 15;
    
    detail[0] = 2; detail[1] = 1;
    detail[2] = 15; detail[3] = 14;
    
    vsf_color(handle, lcolor);
    vsf_interior(handle, 1);
    
    vr_recfl(handle, segment);
    
    vsl_color(handle, dcolor);
    
    for(i=0; i<3; i++)
    {
        v_rbox(handle, detail);
        detail[0] = detail[0] + 2;
        detail[1] = detail[1] + 2;
        detail[2] = detail[2] - 2;
        detail[3] = detail[3] - 2;
    }
}

/* This function clips a section from the upper left corner of the screen
   and puts it into the array pointed to by 'sprite'.                     */

cut_sprite(sprite, screen, width, height)
long *sprite, *screen;
int width, height;
{
    register int row, col, size = 40;
      
    width = (width/2)/4;

    for(row=0; row<height+2; row++)
        for(col=0; col<width; col++)
            *(sprite + width*row + col) = *(screen + size*row + col);
}

/* The following is a little trick to prevent the Megamax linker from
   including some unnecessary stuff. */
   
unlink()
{}
lseek()
{}
fclose()
{}

/**************************************************************************/
/*                    AND THAT'S ALL THERE IS TO THAT!                    */
/**************************************************************************/

