/******************************************************************
     capture.c      MIDI capture program, with playback

                                                  RHO 2/23/89               
******************************************************************/

#define NMAX   2000           /* MIDI storage array size */


#include <stdio.h>
#include <osbind.h>

/******* GLOBAL VARIABLES: *******/

extern long timcnt;       /* msec timer value */
extern int  intr();       /* msec interrupt service routine vector */

struct    mdat {
     char cm;       /* command */
     char fr;       /* pitch/frequency */
     char ve;       /* velocity */
     long ti;       /* time */
} dummy;

struct    mdat      store[NMAX];   /* raw data array*/
struct    mdat      *p;

char b0, b1, b2;
char bnxt;


          
main()
{
     char k;
     int  x;
     int   i, exit;


     clr_struct(store);       /* clear input data array */

     clr_midi();              /* clear MIDI input buffer */

     init_tmr();              /* initialize & start msec timer */

     b0 = 0x90;               /* set default start conditions */
     bnxt = 0;

     printf("\nBegin MIDI input\n");
     p = store;                         /* initialize pointer */

     for( i=0; i<NMAX; i++ ) {          /* input data loop */

          for( x=0, exit=0; ; ) {       /* input wait loop */ 

               if(Bconstat(2)) {        /* test for 'q' exit */
                    x = Bconin(2);    
                    if(x=='q' || x=='Q') {
                         exit = 1;
                         break;
                    }
               }
               if(Bconstat(3)) {        /* get MIDI input byte */
                    k = Bconin(3);    
                    break;
               }
          }

          if(exit)
               break;              /* exit from input data loop */
     

          if(k & 0x80) {           /* store current input byte */
               b0 = k;
               bnxt = 1;
          }
          else if(bnxt == 0 || bnxt == 1) {  
               b1 = k;
               bnxt = 2;
          }
          else if(bnxt == 2) {
               b2 = k;
               bnxt = 0;
          }

          end_c();            /* if end of cmd block,
                                   save cmds in array */
     }  /* end data input loop */

     printf("\nDisplay input data? (Y/N) ");
     x = Bconin(2);
     printf("\n");
     if(( x == 'y') || ( x == 'Y'))
          prt_array(store);             /***  display arrays ***/

     printf("\nPlay it back? (Y/N) ");
     x = Bconin(2);
     printf("\n");
     if(( x == 'y') || ( x == 'Y'))
          pbk(store);

     Jdisint(13);             /* disable timer a interrupt:
                                   required before exit!!     */
}    /* end main */
     
end_c()             /* test for end of cmd block; store data */
{
     int  end_flag;

     end_flag = 0;


     switch(b0 & 0xF0) {      /* (switch uses signed int) */

          case 0xF0:                    /* 1-byte commands: */
                         save(b0, 0, 0, timcnt);
                         end_flag = 1;
                         break;
          case 0xD0:                    /* 2-byte commands: */
          case 0xC0:     
                         if(bnxt == 2) {
                              save(b0, b1, 0, timcnt);
                              end_flag = 1;
                         }
                         if(bnxt == 0) {
                              save(b0, b2, 0, timcnt);
                              end_flag = 1;
                         }
                         break;

          case 0x90:          /* 3-byte command: special case */

                         if(bnxt == 0 && b2 == 0) {
                              save(b0 & 0x8F, b1, b2, timcnt);
                              end_flag = 1;
                              break;
                         }
          case 0x80:                    /* 3-byte commands: */
          case 0xE0:     
          case 0xB0:     
          case 0xA0:     
                         if(bnxt == 0) {
                              save(b0, b1, b2, timcnt);
                              end_flag = 1;
                         }
                         break;

     }  /* end switch */

     return(end_flag);        /* returns 1 if end of cmd block */
}


save(c,f,v,t)       /* store array element & increment pointer */
char c,f,v;
long t;
{
     p->cm = c;
     p->fr = f;
     p->ve = v;
     p->ti = t;
     p++;
}

init_tmr()            /* Initialize Timer A */
{
     timcnt = 0;

     Jdisint(13);                 /* disable timer a interrupt */

     Xbtimer(0x00, 0x04, 49, intr);   /*   set up timer A:
                                     0x04 = prescale divide by 50 
                                       49 = count down value 
                                     intr = interrupt vector 
                                             (ISR address)     */

     Jenabint(13);                 /* enable timer A interrupt */

}


clr_struct(b)             /* clear structure */
char  *b;
{
     int  a, i;

     a = NMAX * (sizeof(dummy));     
     for(i=0; i<a; i++)
          *b++ = 0;
}


clr_midi()     /* clear MIDI input buffer */
{
     while(Bconstat(3))
          Bconin(3);
}


prt_array(pb)         /* display the array contents */
struct mdat  *pb;              
{
     char x;
     int  i, j;

     for(i=0; pb->cm && i<NMAX; i++) {
          printf("cmd   freq  vel   time\n");
          printf("---   ----  ---   ----\n");
          for(j=0; j<10; i++, j++) {
               printf("%02x    ",   (int) 0xFF & pb->cm );
               printf("%02x    ",   pb->fr );
               printf("%02x    ",   pb->ve );
               printf("%Ld \n",     pb->ti );
               pb++;
          }
          x = Bconin(2);    
          if(x=='q' || x=='Q') 
               break;
     }
}



pbk(q)                        /* play back the array contents */
struct  mdat  *q;
{
     int  i;
     long t1, t2, t_old;

     for(i=0;  q->cm && i<NMAX;  i++ ) {

          t1 = timcnt;        
          if(i) {
               do {                        /* output timing wait loop */
                    t2 = timcnt;
               } while ((t2 - t1) < (q->ti - t_old));
          }
          switch(q->cm & 0xF0) {
               case 0xF0:                         /* 1-byte cmds */
                              Bconout(3, q->cm);
                              break;
               case 0xD0:                         /* 2-byte cmds */     
               case 0xC0:     
                              Bconout(3, q->cm);
                              Bconout(3, q->fr);
                              break;

               case 0xE0:                         /* 3-byte cmds */     
               case 0xB0:     
               case 0xA0:     
               case 0x90:     
               case 0x80:     
                              Bconout(3, q->cm);
                              Bconout(3, q->fr);
                              Bconout(3, q->ve);
                              break;

          }  /* end switch */

          t_old = q->ti;          /* save time tag */               
          q++;

     }  /* end for */
}


     
